package org.zeroturnaround.javarebel.integration.spring;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.URL;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.Resource;
import org.springframework.core.io.UrlResource;
import org.springframework.util.ResourceUtils;
import org.zeroturnaround.javarebel.LoggerFactory;

/**
 * @author Rein Raudjärv
 * 
 * @see ResourceUtils
 */
public class ResourceUtil {

  private static final boolean LAST_MODIFIED_FOUND = isLastModifiedFound();

  public static long lastModified(Resource resource) throws IOException {
    if (LAST_MODIFIED_FOUND)
      return resource.lastModified();

    long lastModified = getFileForLastModifiedCheck(resource).lastModified();
    if (lastModified == 0L) {
      throw new FileNotFoundException(resource.getDescription() +
          " cannot be resolved in the file system for resolving its last-modified timestamp");
    }
    return lastModified;
  }

  private static File getFileForLastModifiedCheck(Resource resource) throws IOException {
    if (resource instanceof ClassPathResource)
      return getFileForLastModifiedCheck((ClassPathResource) resource);
    if (resource instanceof UrlResource)
      return getFileForLastModifiedCheck((UrlResource) resource);
    return resource.getFile();
  }

  private static File getFileForLastModifiedCheck(ClassPathResource resource) throws IOException {
    URL url = resource.getURL();
    if (ResourceUtils.isJarURL(url)) {
      URL actualUrl = ResourceUtils.extractJarFileURL(url);
      return ResourceUtils.getFile(actualUrl);
    }
    else {
      return ResourceUtils.getFile(url, resource.getDescription());
    }
  }

  private static File getFileForLastModifiedCheck(UrlResource resource) throws IOException {
    URL url = resource.getURL();
    if (ResourceUtils.isJarURL(url)) {
      URL actualUrl = ResourceUtils.extractJarFileURL(url);
      return ResourceUtils.getFile(actualUrl);
    }
    else {
      return resource.getFile();
    }
  }

  private static boolean isLastModifiedFound() {
    boolean result = false;
    try {
      Resource.class.getDeclaredMethod("lastModified", new Class[0]);
      result = true;
    } catch (SecurityException e) {
    } catch (NoSuchMethodException e) {
    }

    LoggerFactory.getInstance().log("JRebel-Spring: Resource.lastModified() found: " + result);

    return result;
  }

}
